@@ -106,7 +106,7 @@ struct virFDStreamData {
     /* Thread data */
     virThreadPtr thread;
     virCond threadCond;
-    int threadErr;
+    virErrorPtr threadErr;
     bool threadQuit;
     bool threadAbort;
     bool threadDoRead;
@@ -123,6 +123,7 @@ virFDStreamDataDispose(void *obj)
     virFDStreamDataPtr fdst = obj;
 
     VIR_DEBUG("obj=%p", fdst);
+    virFreeError(fdst->threadErr);
     virFDStreamMsgQueueFree(&fdst->msg);
 }
 
@@ -312,6 +313,11 @@ static void virFDStreamEvent(int watch ATTRIBUTE_UNUSED,
         return;
     }
 
+    if (fdst->threadErr) {
+        events |= VIR_STREAM_EVENT_ERROR;
+        virSetError(fdst->threadErr);
+    }
+
     cb = fdst->cb;
     cbopaque = fdst->opaque;
     ff = fdst->ff;
@@ -420,6 +426,8 @@ virFDStreamThreadDoRead(virFDStreamDataPtr fdst,
                         const int fdout,
                         const char *fdinname,
                         const char *fdoutname,
+                        size_t length,
+                        size_t total,
                         size_t *dataLen,
                         size_t buflen)
 {
@@ -433,10 +441,18 @@ virFDStreamThreadDoRead(virFDStreamDataPtr fdst,
         if (virFileInData(fdin, &inData, &sectionLen) < 0)
             goto error;
 
+        if (length &&
+            sectionLen > length - total)
+            sectionLen = length - total;
+
         if (inData)
             *dataLen = sectionLen;
     }
 
+    if (length &&
+        buflen > length - total)
+        buflen = length - total;
+
     if (VIR_ALLOC(msg) < 0)
         goto error;
 
@@ -578,13 +594,6 @@ virFDStreamThread(void *opaque)
     while (1) {
         ssize_t got;
 
-        if (length &&
-            (length - total) < buflen)
-            buflen = length - total;
-
-        if (buflen == 0)
-            break; /* End of requested data from client */
-
         while (doRead == (fdst->msg != NULL) &&
                !fdst->threadQuit) {
             if (virCondWait(&fdst->threadCond, &fdst->parent.lock)) {
@@ -608,6 +617,7 @@ virFDStreamThread(void *opaque)
             got = virFDStreamThreadDoRead(fdst, sparse,
                                           fdin, fdout,
                                           fdinname, fdoutname,
+                                          length, total,
                                           &dataLen, buflen);
         else
             got = virFDStreamThreadDoWrite(fdst, sparse,
@@ -634,7 +644,7 @@ virFDStreamThread(void *opaque)
     return;
 
  error:
-    fdst->threadErr = errno;
+    fdst->threadErr = virSaveLastError();
     goto cleanup;
 }
 
@@ -787,10 +797,10 @@ static int virFDStreamWrite(virStreamPtr st, const char *bytes, size_t nbytes)
     if (fdst->thread) {
         char *buf;
 
-        if (fdst->threadQuit) {
+        if (fdst->threadQuit || fdst->threadErr) {
             virReportSystemError(EBADF, "%s",
                                  _("cannot write to stream"));
-            return -1;
+            goto cleanup;
         }
 
         if (VIR_ALLOC(msg) < 0 ||
@@ -866,7 +876,7 @@ static int virFDStreamRead(virStreamPtr st, char *bytes, size_t nbytes)
         virFDStreamMsgPtr msg = NULL;
 
         while (!(msg = fdst->msg)) {
-            if (fdst->threadQuit) {
+            if (fdst->threadQuit || fdst->threadErr) {
                 if (nbytes) {
                     virReportSystemError(EBADF, "%s",
                                          _("stream is not open"));
@@ -967,6 +977,13 @@ virFDStreamSendHole(virStreamPtr st,
          * the thread to do the lseek() for us. Under no
          * circumstances we can do the lseek() ourselves here. We
          * might mess up file position for the thread. */
+
+        if (fdst->threadQuit || fdst->threadErr) {
+            virReportSystemError(EBADF, "%s",
+                                 _("stream is not open"));
+            goto cleanup;
+        }
+
         if (fdst->threadDoRead) {
             msg = fdst->msg;
             if (msg->type != VIR_FDSTREAM_MSG_TYPE_HOLE) {
@@ -1021,6 +1038,9 @@ virFDStreamInData(virStreamPtr st,
     if (fdst->thread) {
         virFDStreamMsgPtr msg;
 
+        if (fdst->threadErr)
+            goto cleanup;
+
         while (!(msg = fdst->msg)) {
             if (fdst->threadQuit) {
                 *inData = *length = 0;
